@@ -85,7 +85,7 @@ class GoGridNode(Node):
     # so uuid of node should not change after add is completed
     def get_uuid(self):
         return hashlib.sha1(
-            b("%s:%d" % (self.public_ips, self.driver.type))
+            b("%s:%s" % (self.public_ips, self.driver.type))
         ).hexdigest()
 
 
@@ -103,6 +103,12 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
 
     _instance_types = GOGRID_INSTANCE_TYPES
 
+    def __init__(self, *args, **kwargs):
+        """
+        @inherits: :class:`NodeDriver.__init__`
+        """
+        super(GoGridNodeDriver, self).__init__(*args, **kwargs)
+
     def _get_state(self, element):
         try:
             return STATE[element['state']['name']]
@@ -163,6 +169,10 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
         return images
 
     def list_nodes(self):
+        """
+        @inherits: :class:`NodeDriver.list_nodes`
+        :rtype: ``list`` of :class:`GoGridNode`
+        """
         passwords_map = {}
 
         res = self._server_list()
@@ -182,6 +192,10 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
                 for el in res['list']]
 
     def reboot_node(self, node):
+        """
+        @inherits: :class:`NodeDriver.reboot_node`
+        :type node: :class:`GoGridNode`
+        """
         id = node.id
         power = 'restart'
         res = self._server_power(id, power)
@@ -190,6 +204,10 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
         return True
 
     def destroy_node(self, node):
+        """
+        @inherits: :class:`NodeDriver.reboot_node`
+        :type node: :class:`GoGridNode`
+        """
         id = node.id
         res = self._server_delete(id)
         if not res.success():
@@ -240,29 +258,29 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
         """Don't block until GoGrid allocates id for a node
         but return right away with id == None.
 
-        The existance of this method is explained by the fact
+        The existence of this method is explained by the fact
         that GoGrid assigns id to a node only few minutes after
         creation.
 
 
-        @keyword    name:   String with a name for this new node (required)
-        @type       name:   C{str}
+        :keyword    name:   String with a name for this new node (required)
+        :type       name:   ``str``
 
-        @keyword    size:   The size of resources allocated to this node .
+        :keyword    size:   The size of resources allocated to this node .
                             (required)
-        @type       size:   L{NodeSize}
+        :type       size:   :class:`NodeSize`
 
-        @keyword    image:  OS Image to boot on node. (required)
-        @type       image:  L{NodeImage}
+        :keyword    image:  OS Image to boot on node. (required)
+        :type       image:  :class:`NodeImage`
 
-        @keyword    ex_description: Description of a Node
-        @type       ex_description: C{str}
+        :keyword    ex_description: Description of a Node
+        :type       ex_description: ``str``
 
-        @keyword    ex_ip: Public IP address to use for a Node. If not
+        :keyword    ex_ip: Public IP address to use for a Node. If not
             specified, first available IP address will be picked
-        @type       ex_ip: C{str}
+        :type       ex_ip: ``str``
 
-        @rtype: L{Node}
+        :rtype: :class:`GoGridNode`
         """
         name = kwargs['name']
         image = kwargs['image']
@@ -287,14 +305,16 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
     def create_node(self, **kwargs):
         """Create a new GoGird node
 
-        @inherits: L{NodeDriver.create_node}
+        @inherits: :class:`NodeDriver.create_node`
 
-        @keyword    ex_description: Description of a Node
-        @type       ex_description: C{str}
+        :keyword    ex_description: Description of a Node
+        :type       ex_description: ``str``
 
-        @keyword    ex_ip: Public IP address to use for a Node. If not
+        :keyword    ex_ip: Public IP address to use for a Node. If not
                     specified, first available IP address will be picked
-        @type       ex_ip: C{str}
+        :type       ex_ip: ``str``
+
+        :rtype: :class:`GoGridNode`
         """
         node = self.ex_create_node_nowait(**kwargs)
 
@@ -327,13 +347,13 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
 
         http://wiki.gogrid.com/wiki/index.php/MyGSI
 
-        @keyword    node: node to use as a base for image
-        @type       node: L{Node}
+        :keyword    node: node to use as a base for image
+        :type       node: :class:`GoGridNode`
 
-        @keyword    name: name for new image
-        @type       name: C{str}
+        :keyword    name: name for new image
+        :type       name: ``str``
 
-        @rtype: L{NodeImage}
+        :rtype: :class:`NodeImage`
         """
         params = {'server': node.id,
                   'friendlyName': name}
@@ -345,16 +365,16 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
     def ex_edit_node(self, **kwargs):
         """Change attributes of a node.
 
-        @keyword    node: node to be edited
-        @type       node: L{Node}
+        :keyword    node: node to be edited (required)
+        :type       node: :class:`GoGridNode`
 
-        @keyword    size: new size of a node
-        @type       size: L{NodeSize}
+        :keyword    size: new size of a node (required)
+        :type       size: :class:`NodeSize`
 
-        @keyword    ex_description: new description of a node
-        @type       ex_description: C{str}
+        :keyword    ex_description: new description of a node
+        :type       ex_description: ``str``
 
-        @rtype: L{Node}
+        :rtype: :class:`Node`
         """
         node = kwargs['node']
         size = kwargs['size']
@@ -373,19 +393,19 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
     def ex_edit_image(self, **kwargs):
         """Edit metadata of a server image.
 
-        @keyword    image: image to be edited
-        @type       image: L{NodeImage}
+        :keyword    image: image to be edited (required)
+        :type       image: :class:`NodeImage`
 
-        @keyword    public: should be the image public?
-        @type       public: C{bool}
+        :keyword    public: should be the image public (required)
+        :type       public: ``bool``
 
-        @keyword    ex_description: description of the image (optional)
-        @type       ex_description: C{str}
+        :keyword    ex_description: description of the image (optional)
+        :type       ex_description: ``str``
 
-        @keyword    name: name of the image
-        @type       name C{str}
+        :keyword    name: name of the image
+        :type       name ``str``
 
-        @rtype: L{NodeImage}
+        :rtype: :class:`NodeImage`
         """
 
         image = kwargs['image']
@@ -409,22 +429,22 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
         """Return list of IP addresses assigned to
         the account.
 
-        @keyword    public: set to True to list only
+        :keyword    public: set to True to list only
                     public IPs or False to list only
                     private IPs. Set to None or not specify
                     at all not to filter by type
-        @type       public: C{bool}
+        :type       public: ``bool``
 
-        @keyword    assigned: set to True to list only addresses
+        :keyword    assigned: set to True to list only addresses
                     assigned to servers, False to list unassigned
                     addresses and set to None or don't set at all
                     not no filter by state
-        @type       assigned: C{bool}
+        :type       assigned: ``bool``
 
-        @keyword    location: filter IP addresses by location
-        @type       location: L{NodeLocation}
+        :keyword    location: filter IP addresses by location
+        :type       location: :class:`NodeLocation`
 
-        @rtype: C{list} of L{GoGridIpAddress}
+        :rtype: ``list`` of :class:`GoGridIpAddress`
         """
 
         params = {}
